@@ -1,6 +1,8 @@
 import os
 import sys
 import traceback
+import typing
+
 from asyncio import get_event_loop
 from importlib import import_module
 from inspect import isclass, getfile, getmodulename
@@ -8,6 +10,9 @@ from pathlib import Path
 from traceback import print_exc
 from typing import Dict, Callable, Any
 
+if typing.TYPE_CHECKING:
+    from .poll import PollData
+
 from .channel import Channel
 from .command import Command
 from .config import cfg
@@ -166,6 +171,20 @@ class Mod:
         triggered when a viewers redeems channel points for a reward
         """
 
+    async def on_poll_started(self, channel: Channel, poll: 'PollData'):
+        """
+        triggered when a poll starts
+        :param channel: channel the poll originated in
+        :param poll: the poll that was started
+        """
+
+    async def on_poll_ended(self, channel: Channel, poll: 'PollData'):
+        """
+        triggered when a poll ends
+        :param channel: channel the poll originated in
+        :param poll: the poll that has ended
+        """
+
     # endregion
 
 
